@@ -1,23 +1,21 @@
-#' Sensitivity of NNET models
+#' Sensitivity of MLP models
 #'
 #' @description Function for evaluating the sensitivities of the inputs
 #'   variables in a mlp model
 #' @param MLP.fit fitted neural network model
-#' @param trData \code{data.frame} containing the data to evaluate the
-#'   sensitivity of the model
-#' @param actfunc \code{character} vector indicating the activation function of
-#'   each neurons layer.
-#' @param .returnSens \code{logical} value. If \code{TRUE}, sensitivity of the
-#'   model is returned.
+#' @param trData \code{data.frame} containing the data to evaluate the sensitivity of the model
+#' @param actfunc \code{character} vector indicating the activation function of each
+#'   neurons layer.
+#' @param deractfunc \code{character} vector indicating the derivative of the activation
+#' function of each neurons layer.
+#' @param .returnSens DEPRECATED
 #' @param preProc preProcess structure applied to the training data. See also
 #'   \code{\link[caret]{preProcess}}
 #' @param terms function applied to the training data to create factors. See
 #'   also \code{\link[caret]{train}}
 #' @param plot \code{logical} whether or not to plot the analysis. By default is
 #'   \code{TRUE}.
-#' @param .rawSens \code{logical} whether or not to return the sensitivity of
-#'   each row of the data provided, or return the mean, sd and mean of the
-#'   square of the sensitivities. By default is \code{FALSE}.
+#' @param .rawSens DEPRECATED
 #' @param output_name \code{character} name of the output variable in order to
 #'   avoid changing the name of the output variable in \code{trData} to
 #'   '.outcome'
@@ -35,10 +33,8 @@
 #'   is of the output (\code{FALSE}) or from the input (\code{TRUE}) of the
 #'   \code{sens_end_layer} layer of the model. By default is \code{FALSE}.
 #' @param ...	additional arguments passed to or from other methods
-#' @return dataframe with the sensitivities obtained for each variable if
-#'   \code{.returnSens = TRUE}. If \code{.returnSens = FALSE}, the sensitivities
-#'   without processing are returned in a 3D array. If there is more than one
-#'   output, the sensitivities of each output are given in a list.
+#' @return \code{SensMLP} object with the sensitivity metrics and sensitivities of
+#' the MLP model passed to the function.
 #' @section Plots: \itemize{ \item Plot 1: colorful plot with the classification
 #'   of the classes in a 2D map \item Plot 2: b/w plot with probability of the
 #'   chosen class in a 2D map \item Plot 3: plot with the stats::predictions of
@@ -197,10 +193,10 @@
 #'
 #'
 #' ## USE DEFAULT METHOD ----------------------------------------------------------
-#' NeuralSens::SensAnalysisMLP(RegNNET$caret$finalModel$wts,
+#' NeuralSens::SensAnalysisMLP(caretmod$finalModel$wts,
 #'                             trData = fdata.Reg.tv,
-#'                             mlpstr = RegNNET$caret$finalModel$n,
-#'                             coefnames = RegNNET$caret$coefnames,
+#'                             mlpstr = caretmod$finalModel$n,
+#'                             coefnames = caretmod$coefnames,
 #'                             actfun = c("linear","sigmoid","linear"),
 #'                             output_name = "DEM")
 #'
@@ -365,9 +361,11 @@ SensAnalysisMLP.default <- function(MLP.fit,
                                     sens_end_input = FALSE,
                                     trData,
                                     actfunc = NULL,
+                                    deractfunc = NULL,
                                     preProc = NULL,
                                     terms = NULL,
-                                    output_name = NULL,  ...) {
+                                    output_name = NULL,
+                                    ...) {
   ### Things needed for calculating the sensibilities:
   #   - Structure of the model  -> MLP.fit$n
   #   - Weights of the model    -> MLP.fit$wts
@@ -377,33 +375,6 @@ SensAnalysisMLP.default <- function(MLP.fit,
   # Obtain structure of fitted model
   mlpstr <- MLP.fit$n
 
-  # Check all sensitivity arguments makes sense
-  # Check which is the output we want the derivative of
-  if (sens_end_layer == "last") {
-    # User wants the derivative of the last layer of the neural network
-    sens_end_layer = length(mlpstr)
-  }
-
-  # Detect that origin and end layer are defined by a number
-  if (!is.numeric(c(sens_end_layer,sens_origin_layer))) {
-    stop("End layer and origin layer must be specified by a strictly positive number")
-  }
-  # Detect that layers are specified by strictly positive numbers
-  if (any(c(sens_end_layer,sens_origin_layer) <= 0)) {
-    stop("End layer and origin layer must be specified by a strictly positive number")
-  }
-  # Detect that at least there is one layer between origin and end of derivatives
-  if ((sens_end_layer < sens_origin_layer) ||
-      ((sens_end_layer == sens_origin_layer) &&
-       !(sens_origin_input && !sens_end_input))) {
-    stop("There must be at least one layer between end and origin")
-  }
-
-  # Detect that exists the layers specified
-  if (sens_end_layer > length(mlpstr)) {
-    stop("The layers specified could not be found in the neural network model")
-  }
-
   # Obtain weights
   nwts <- NeuralNetTools::neuralweights(MLP.fit$wts, struct = mlpstr)
   wts <- nwts$wts
@@ -468,130 +439,99 @@ SensAnalysisMLP.default <- function(MLP.fit,
   D <- list()
 
   # Initialize the activation and the derivative of the activation function for each layer
+  if (is.null(deractfunc)) deractfunc <- actfunc
   ActivationFunction <- lapply(actfunc, NeuralSens::ActFunc)
-  DerActivationFunction <- lapply(actfunc, NeuralSens::DerActFunc)
+  DerActivationFunction <- lapply(deractfunc, NeuralSens::DerActFunc)
 
   W[[1]] <- diag(ncol(TestData)+1)
   # For each row in the TestData
   Z[[1]] <- as.matrix(TestData)
   O[[1]] <- ActivationFunction[[1]](Z[[1]])
-
+  D[[1]] <- array(diag(mlpstr[1]),
+                  dim=c(mlpstr[1],
+                        mlpstr[1],
+                        nrow(TestData)))
+  for (irow in 1:nrow(TestData)) {
+    D[[1]][,,irow] <- DerActivationFunction[[1]](Z[[1]][irow,])
+  }
   # For each layer, calculate the input to the activation functions of each layer
   # This inputs are gonna be used to calculate the derivatives and the output of each layer
   for (l in 2:length(mlpstr)){
     W[[l]] <- data.matrix(as.data.frame(wts[(sum(mlpstr[1:(l-1)])-mlpstr[1]+1):(sum(mlpstr[1:l])-mlpstr[1])]))
     Z[[l]] <- cbind(1, O[[l-1]]) %*% W[[l]]
     O[[l]] <- ActivationFunction[[l]](Z[[l]])
+    D[[l]] <- array(diag(mlpstr[l]),
+                    dim=c(mlpstr[l],
+                          mlpstr[l],
+                          nrow(TestData)))
+    for (irow in 1:nrow(TestData)) {
+      D[[l]][,,irow] <- DerActivationFunction[[l]](Z[[l]][irow,])
+    }
   }
 
-  # Check if we want all the derivatives to be returned
   args <- list(...)
 
   if (!"return_all_sens" %in% names(args[[1]])) {
-    # Calculate derivatives
-    D[[1]] <- array(diag(mlpstr[sens_origin_layer]),
-                    dim=c(mlpstr[sens_origin_layer],
-                          mlpstr[sens_origin_layer],
-                          nrow(TestData)))
-    if (sens_origin_input) {
-      for (irow in 1:nrow(TestData)) {
-        D[[1]][,,irow] <- DerActivationFunction[[sens_origin_layer]](Z[[sens_origin_layer]][irow,])
-      }
-    }
-    l <- 1
-    # Only perform further operations if origin is not equal to end layer
-    if (sens_origin_layer != sens_end_layer) {
-      counter <- 1
-      for (l in (sens_origin_layer+1):sens_end_layer) {
-        counter <- counter + 1
-        D[[counter]] <- array(NA, dim=c(mlpstr[sens_origin_layer], mlpstr[l], nrow(TestData)))
-        # Check if it must be multiplied by the jacobian of the nest layer
-        if ((l == sens_end_layer) && sens_end_input) {
-          for (irow in 1:nrow(TestData)){
-            D[[counter]][,,irow] <- D[[counter - 1]][,,irow] %*% W[[l]][2:nrow(W[[l]]),]
-          }
-        } else {
-          for (irow in 1:nrow(TestData)){
-            D[[counter]][,,irow] <- D[[counter - 1]][,,irow] %*% W[[l]][2:nrow(W[[l]]),] %*% DerActivationFunction[[l]](Z[[l]][irow,])
-          }
-        }
-      }
-      l <- counter
-    }
+    # out <- structure(list(
+    #   sens = NULL,
+    #   raw_sens = NULL,
+    #   layer_derivatives = D,
+    #   mlp_struct = mlpstr,
+    #   mlp_wts = W,
+    #   layer_origin = sens_origin_layer,
+    #   layer_origin_input = sens_origin_input,
+    #   layer_end = sens_end_layer,
+    #   layer_end_input = sens_end_input,
+    #   trData = trData,
+    #   coefnames = varnames,
+    #   output_name = output_name
+    # ),
+    # class = "SensMLP")
+    out <- list(
+      sens = NULL,
+      raw_sens = NULL,
+      layer_derivatives = D,
+      mlp_struct = mlpstr,
+      mlp_wts = W,
+      layer_origin = sens_origin_layer,
+      layer_origin_input = sens_origin_input,
+      layer_end = sens_end_layer,
+      layer_end_input = sens_end_input,
+      trData = trData,
+      coefnames = varnames,
+      output_name = output_name
+    )
 
+    out <- ComputeSensMeasures(out)
 
-    # Prepare the derivatives for the following calculations
-    der <- aperm(D[[l]],c(3,1,2))
-    if (sens_origin_layer != 1) {
-      varnames = paste0("Neuron ",sens_origin_layer,1:mlpstr[sens_origin_layer])
-    }
-    colnames(der) <- varnames
-    # Obtain sensitivities of the first output and create plots if required
-    sens <-
-      data.frame(
-        varNames = varnames,
-        mean = colMeans(der[, , 1], na.rm = TRUE),
-        std = apply(der[, , 1], 2, stats::sd, na.rm = TRUE),
-        meanSensSQ = colMeans(der[, , 1] ^ 2, na.rm = TRUE)
-      )
-    rownames(sens) <- NULL
+    out <- SensMLP(
+      out$sens,
+      out$raw_sens,
+      mlpstr,
+      trData,
+      varnames,
+      names(out$sens)
+    )
 
     if (plot) {
       # show plots if required
-      NeuralSens::SensitivityPlots(sens,der = der[,,1])
-    }
-
-    if (.returnSens) {
-      if(!.rawSens) {
-        # Check if there are more than one output and return a list
-        # with the sensitivities of each output. If not, return a data.frame
-        if (dim(der)[3] > 1) {
-          sens <- list(sens)
-          for (i in 2:dim(der)[3]) {
-            sens[[i]] <- data.frame(
-              varNames = varnames,
-              mean = colMeans(der[, , i], na.rm = TRUE),
-              std = apply(der[, , i], 2, stats::sd, na.rm = TRUE),
-              meanSensSQ = colMeans(der[, , i] ^ 2, na.rm = TRUE)
-            )
-            rownames(sens[[i]]) <- NULL
-          }
-          if (is.factor(trData$.outcome)) {
-            names(sens) <- make.names(unique(trData$.outcome), unique = TRUE)
-          } else if (!is.null(output_name)) {
-            if (length(output_name) > 2) {
-              names(sens <- output_name)
-            }
-          }
-        }
-        return(sens)
-      } else {
-        # Return sensitivities without processing
-        if (mlpstr[length(mlpstr)] > 1) {
-          if (is.factor(trData$.outcome)) {
-            names(sens) <- make.names(unique(trData$.outcome), unique = TRUE)
-          } else if (!is.null(output_name)) {
-            if (length(output_name) > 2) {
-              names(sens <- output_name)
-            }
-          }
-        } else {
-          if (!is.null(output_name)) {
-            dimnames(der)[[3]] <- output_name
-          }
-        }
-        return(der)
+      args <- list(...)
+      zoom <- TRUE
+      quit.legend <- FALSE
+      der <- TRUE
+      if ("zoom" %in% names(args[[1]])) {
+        zoom <- args[[1]]$zoom
       }
+      if ("quit.legend" %in% names(args[[1]])) {
+        quit.legend <- args[[1]]$quit.legend
+      }
+      if ("der" %in% names(args[[1]])) {
+        der <- args[[1]]$der
+      }
+      NeuralSens::SensitivityPlots(out, der, zoom, quit.legend)
     }
+    return(out)
   } else {
-    # For each layer, calculate the input to the activation functions of each layer
-    # This inputs are gonna be used to calculate the derivatives and the output of each layer
-    for (l in 1:length(mlpstr)){
-      D[[l]]<- array(NA, dim=c(mlpstr[l], mlpstr[l], nrow(TestData)))
-      for(irow in 1:nrow(TestData)){
-        D[[l]][,,irow] <- DerActivationFunction[[l]](Z[[l]][irow,])
-      }
-    }
 
     # Calculate derivatives with respect with the last layer's output
     d <- list()
@@ -805,8 +745,9 @@ SensAnalysisMLP.H2OMultinomialModel <- function(MLP.fit,
                           preProc = preProc,
                           terms = NULL,
                           plot = plot,
-                          output_name = MLP.fit@parameters$y,
-                          args[!names(args) %in% c("trData")])
+                          output_name = if("output_name" %in% names(args)){args$output_name}else{MLP.fit@parameters$y},
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("trData","output_name","deractfunc")])
   }
 
 #' @rdname SensAnalysisMLP
@@ -966,8 +907,9 @@ SensAnalysisMLP.H2ORegressionModel <- function(MLP.fit,
                           preProc = preProc,
                           terms = NULL,
                           plot = plot,
-                          output_name = MLP.fit@parameters$y,
-                          args[!names(args) %in% c("trData")])
+                          output_name = if("output_name" %in% names(args)){args$output_name}else{MLP.fit@parameters$y},
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("trData","output_name","deractfunc")])
   }
 
 
@@ -1038,7 +980,8 @@ SensAnalysisMLP.list <- function(MLP.fit,
                           terms = NULL,
                           plot = plot,
                           output_name = if("output_name" %in% names(args)){args$output_name}else{".outcome"},
-                          args[!names(args) %in% c("output_name")])
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("output_name","deractfunc")])
 }
 
 #' @rdname SensAnalysisMLP
@@ -1131,7 +1074,8 @@ SensAnalysisMLP.mlp <- function(MLP.fit,
                           terms = terms,
                           plot = plot,
                           output_name = if("output_name" %in% names(args)){args$output_name}else{".outcome"},
-                          args[!names(args) %in% c("output_name")])
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("output_name","deractfunc")])
 }
 
 #' @rdname SensAnalysisMLP
@@ -1185,7 +1129,8 @@ SensAnalysisMLP.nn <- function(MLP.fit,
                             terms = terms,
                             plot = FALSE,
                             output_name = names(trData)[names(trData) == MLP.fit$model.list$response],
-                            args[!names(args) %in% c("output_name")])
+                            deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                            args[!names(args) %in% c("deractfunc")])
     sensit[((j-1)*nrow(trData)+1):(j*nrow(trData)),,] <- sensitivities
   }
   colnames(sensit) <- finalModel$coefnames
@@ -1262,7 +1207,8 @@ SensAnalysisMLP.nnet <- function(MLP.fit,
                           terms = terms,
                           plot = plot,
                           output_name = if("output_name" %in% names(args)){args$output_name}else{".outcome"},
-                          args[!names(args) %in% c("output_name")])
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("output_name","deractfunc")])
 }
 
 #' @rdname SensAnalysisMLP
@@ -1367,6 +1313,7 @@ SensAnalysisMLP.nnetar <- function(MLP.fit,
     sensitivities[[i]] <-  SensAnalysisMLP.default(finalModel,
                                               trData = trData,
                                               actfunc = actfun,
+                                              deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
                                               .returnSens = TRUE,
                                               .rawSens = TRUE,
                                               sens_origin_layer = sens_origin_layer,
@@ -1420,6 +1367,7 @@ SensAnalysisMLP.numeric <- function(MLP.fit,
                                     sens_origin_input = TRUE,
                                     sens_end_input = FALSE,
                                     trData,
+                                    actfunc = NULL,
                                     preProc = NULL,
                                     terms = NULL,
                                     ...) {
@@ -1434,7 +1382,17 @@ SensAnalysisMLP.numeric <- function(MLP.fit,
   }
   finalModel$n <- args$mlpstr
   # Define the names of the explanatory variables
-  if (!(".outcome" %in% names(trData))) {
+  if ((".outcome" %in% names(trData)) || "output_name" %in% names(args)) {
+    if (!("coefnames" %in% names(args))) {
+      if ("output_name" %in% names(args)) {
+        finalModel$coefnames <- names(trData)[names(trData) != args$output_name]
+      } else {
+        finalModel$coefnames <- names(trData)[names(trData) != ".outcome"]
+      }
+    } else {
+      finalModel$coefnames <- args$coefnames
+    }
+  } else {
     if (!("coefnames" %in% names(args))) {
       stop("Names of explanatory variables must be passed in coefnames argument")
     }
@@ -1442,24 +1400,16 @@ SensAnalysisMLP.numeric <- function(MLP.fit,
     if (!all(args$coefnames %in% names(trData))) {
       stop("Explanatory variables defined in coefnames has not been found in trData")
     }
-  } else {
-    if (!("coefnames" %in% names(args))) {
-      finalModel$coefnames <- names(trData)[names(trData) != ".outcome"]
-    } else {
-      finalModel$coefnames <- args$coefnames
-    }
   }
   # Define the activation functions used in the neural network
   # The activation functions must be passed as actfun argument
-  if (!"actfun" %in% names(args)) {
-    stop("The activation functions used in the MLP must be passed as actfun argument")
-  } else if (length(args$actfun) != length(args$mlpstr)) {
+  if (length(actfunc) != length(args$mlpstr)) {
     stop("Number of activation functions does not match the structure of the MLP")
   }
 
   SensAnalysisMLP.default(finalModel,
                           trData = trData,
-                          actfunc = args$actfun,
+                          actfunc = actfunc,
                           .returnSens = .returnSens,
                           .rawSens = .rawSens,
                           sens_origin_layer = sens_origin_layer,
@@ -1470,5 +1420,6 @@ SensAnalysisMLP.numeric <- function(MLP.fit,
                           terms = terms,
                           plot = plot,
                           output_name = if("output_name" %in% names(args)){args$output_name}else{".outcome"},
-                          ...)
+                          deractfunc = if("deractfunc" %in% names(args)){args$deractfunc}else{NULL},
+                          args[!names(args) %in% c("output_name","deractfunc")])
 }
